# pip install python-docx

import time
from platform import node
from datetime import datetime as dt
from subprocess import check_output

from docx import Document


def get_size(bts: int, ending='iB') -> str:
    size = 1024
    for item in ["", "K", "M", "G", "T", "P"]:
        if bts < size:
            return f"{bts:.2f} {item}{ending}" if bts > 0 else f"{bts:.2f} {item}B"
        bts /= size


# OS
def wmic_os() -> (dict, bool):
    os_info = dict()
    try:
        os_i = check_output("wmic OS get Caption, InstallDate, Version, WindowsDirectory, LastBootUpTime /value",
                            shell=True).decode().strip()
    except UnicodeDecodeError:
        os_i = check_output("wmic OS get Caption, InstallDate, Version, WindowsDirectory, LastBootUpTime /value",
                            shell=True).decode("cp866").strip()
    if os_i:
        out = [{x.strip().split("=")[0]: x.strip().split("=")[1]} for x in os_i.splitlines() if x.strip()]
        if comp_name := check_output("wmic computersystem get name /value", shell=True).decode().strip():
            os_info.update({"ComputerName": comp_name.split("=")[1].strip()})
        for item in out:
            for it in item:
                if item[it]:
                    if it == "InstallDate":
                        date = dt.strptime(item[it][:len(item[it]) - 4], '%Y%m%d%H%M%S.%f').strftime(
                            "%Y-%m-%d %H:%M:%S")
                        item = {"InstallDate": date}
                    if it == "LastBootUpTime":
                        date = dt.strptime(item[it][:len(item[it]) - 4], '%Y%m%d%H%M%S.%f').strftime(
                            "%Y-%m-%d %H:%M:%S")
                        item = {"LastBootUpTime": date}
                    os_info.update(item)
    if tz_get := check_output("wmic TIMEZONE get Caption /value", shell=True).decode().strip():
        tz = [{"TimeZone": x.strip().split("=")[1]} for x in tz_get.splitlines() if x.strip()]
        os_info.update(tz[0])
    if user_get := check_output('wmic USERACCOUNT Where (Status="OK") get Name /value', shell=True).decode().strip():
        user = [{"UserName": x.strip().split("=")[1]} for x in user_get.splitlines() if x.strip()]
        os_info.update(user[0])
    return os_info if os_info else False


# BIOS
def bios_wmic() -> (dict, bool):
    bios_info = dict()
    if bios := check_output("wmic BIOS get Version, Manufacturer, Name /value", shell=True).decode(). \
            strip():
        out = [{f'{x.strip().split("=")[0]}': x.strip().split("=")[1]} for x in bios.splitlines() if x.strip()]
        for item in out:
            bios_info.update(item)
        return bios_info if bios_info else False
    return False


# Motherboard
def motherboard_wmic() -> (dict, bool):
    mb_info = dict()
    if mb_get := check_output("wmic baseboard get Product, Manufacturer /value", shell=True).decode().strip():
        out = [{f'{x.strip().split("=")[0]}': x.strip().split("=")[1]} for x in mb_get.splitlines() if x.strip()]
        for item in out:
            mb_info.update(item)
        return mb_info if mb_info else False
    return False


# CPU
def cpu_wmic() -> (dict, bool):
    mb_info = dict()
    if mb_get := check_output("wmic cpu get Name, Caption, Manufacturer, SocketDesignation, MaxClockSpeed /value",
                              shell=True).decode().strip():
        out = [{f'{x.strip().split("=")[0]}': x.strip().split("=")[1]} for x in mb_get.splitlines() if x.strip()]
        for item in out:
            mb_info.update(item)
    if core_all := check_output("wmic COMPUTERSYSTEM get NumberOfLogicalProcessors /value",
                                shell=True).decode().strip():
        mb_info.update({'NumberOfPhysicalProcessors': str(int(int(core_all.split("=")[1].strip()) / 2)),
                        f'{core_all.split("=")[0].strip()}': core_all.split("=")[1].strip()})
    return mb_info if mb_info else False


# GPU
def gpu_wmic() -> (dict, bool):
    gpu = dict()
    if output := check_output('wmic path win32_VideoController get Name, AdapterRAM, VideoProcessor, '
                              'CurrentHorizontalResolution, CurrentVerticalResolution, CurrentRefreshRate /value',
                              shell=True).decode().strip():
        out = [{x.strip().split("=")[0]: x.strip().split("=")[1]} for x in output.splitlines() if x.strip()]
        res = ""
        for vrm in out:
            for vr in vrm:
                if vr == 'AdapterRAM':
                    gpu.update({"AdapterRAM": get_size(int(vrm[vr]))})
                    continue
                if vr == 'CurrentHorizontalResolution':
                    res += f'{vrm[vr]}x'
                    continue
                if vr == 'CurrentVerticalResolution':
                    res += str(vrm[vr])
                    gpu.update({"Resolution": res})
                    continue
                gpu.update(vrm)
        return gpu if gpu else False
    return False


# Memory
def memory_wmic() -> (dict, bool):
    memory_info = dict()
    if total_memory_get := check_output("wmic COMPUTERSYSTEM get TotalPhysicalMemory /value",
                                        shell=True).decode().strip():
        total_memory = [{x.strip().split("=")[0]: get_size(int(x.strip().split("=")[1]))}
                        for x in total_memory_get.splitlines() if x.strip()]
        memory_info.update(total_memory[0])
    if memory_tag_get := check_output("wmic MEMORYCHIP get Tag /value", shell=True).decode().strip():
        memory_tag = [x.strip().split("=")[1] for x in memory_tag_get.splitlines() if x.strip()]
        for tag in memory_tag:
            info_get = check_output(f'wmic MEMORYCHIP Where (Tag="{tag}") get Capacity, ConfiguredClockSpeed, '
                                    f'Manufacturer, PartNumber, SerialNumber /value', shell=True).decode().strip()
            memory_info[tag.strip()] = dict()
            for x in info_get.splitlines():
                if x.strip():
                    if x.strip().split("=")[0] == "Capacity" and x.strip().split("=")[1]:
                        memory_info[tag.strip()].update({
                            x.strip().split("=")[0].strip(): get_size(int(x.strip().split("=")[1])).strip()
                        })
                    else:
                        memory_info[tag.strip()].update({
                            x.strip().split("=")[0].strip(): x.strip().split("=")[1].strip()
                        })
    return memory_info if memory_info else False


# HDD, SSD
def hdd_ssd_wmic() -> (dict, bool):
    disk_info = dict()
    if caption_get := check_output("wmic diskdrive get Caption /value", shell=True).decode().strip():
        caption = [x.strip().split("=")[1] for x in caption_get.splitlines() if x.strip()]
        for num, cap in enumerate(caption):
            if info_get := check_output(
                    f'wmic diskdrive Where (Caption="{cap}") get DeviceID, FirmwareRevision, MediaType, '
                    f'Partitions, SerialNumber, Size /value', shell=True).decode().strip():
                disk_info[f'{cap} {num}'] = dict()
                disk_info[f'{cap} {num}'].update({"Product": cap})
                for x in info_get.splitlines():
                    if x.strip():
                        if x.strip().split("=")[0] == "Size" and x.strip().split("=")[1]:
                            disk_info[f'{cap} {num}'].update({
                                f'{x.strip().split("=")[0].strip()}': get_size(int(x.strip().split("=")[1].strip()))
                            })
                        else:
                            disk_info[f'{cap} {num}'].update({
                                f'{x.strip().split("=")[0].strip()}': x.strip().split("=")[1].strip()
                            })
    return disk_info if disk_info else False


# CD-ROM
def cdrom_wmic() -> (dict, bool):
    cdrom_info = dict()
    if caption_get := check_output("wmic CDROM get Caption /value", shell=True).decode().strip():
        caption = [x.strip().split("=")[1] for x in caption_get.splitlines() if x.strip()]
        for num, cap in enumerate(caption):
            if info_get := check_output(
                    f'wmic CDROM Where (Caption="{cap}") get Drive, VolumeName, VolumeSerialNumber, Size '
                    f'/value', shell=True).decode().strip():
                cdrom_info[cap] = dict()
                cdrom_info[cap].update({"Product": cap})
                for x in info_get.splitlines():
                    if x.strip():
                        if x.strip().split("=")[0].strip() == "Size" and x.strip().split("=")[1].strip():
                            cdrom_info[cap].update({
                                f'{x.strip().split("=")[0].strip()}': get_size(int(x.strip().split("=")[1].strip()))
                            })
                        cdrom_info[cap].update({
                            f'{x.strip().split("=")[0].strip()}': x.strip().split("=")[1].strip()
                        })
    return cdrom_info if cdrom_info else False


# Network Interface
def nic_wmic() -> (dict, bool):
    nic_info = dict()
    if description_get := check_output('wmic NIC Where (PhysicalAdapter="TRUE") get Description /value', shell=True). \
            decode().strip():
        description = [x.strip().split("=")[1] for x in description_get.splitlines() if x.strip()]
        for nic in description:
            try:
                nic_d = check_output(
                    f'wmic NIC Where (Description="{nic}") get MACAddress, Manufacturer, NetConnectionID'
                    f' /value', shell=True).decode().strip()
            except UnicodeDecodeError:
                nic_d = check_output(
                    f'wmic NIC Where (Description="{nic}") get MACAddress, Manufacturer, NetConnectionID'
                    f' /value', shell=True).decode("cp866").strip()
            try:
                nicconfig_d = check_output(
                    f'wmic NICCONFIG Where (Description="{nic}") get DHCPServer, IPAddress /value',
                    shell=True).decode().strip()
            except UnicodeDecodeError:
                nicconfig_d = check_output(
                    f'wmic NICCONFIG Where (Description="{nic}") get DHCPServer, IPAddress /value',
                    shell=True).decode("cp866").strip()
            nic_info[nic.strip()] = dict()
            nic_info[nic.strip()].update({"Product": nic.strip()})
            if nic_d:
                for x in nic_d.splitlines():
                    if x.strip():
                        nic_info[nic.strip()].update({
                            x.strip().split("=")[0].strip(): x.strip().split("=")[1].strip()
                        })
            if nicconfig_d:
                for x in nicconfig_d.splitlines():
                    if x.strip():
                        nic_info[nic.strip()].update({
                            x.strip().split("=")[0].strip(): x.strip().split("=")[1].strip()
                        })
    return nic_info if nic_info else False


wmic_info = ""


def print_wmic(part, dict_info):
    global wmic_info
    synonyms = {"ComputerName": "Имя компьютера", "Caption": "Название", "InstallDate": "Дата установки",
                "LastBootUpTime": "Время последней загрузки", "Version": "Версия",
                "WindowsDirectory": "Директория Windows", "TimeZone": "Часовой пояс", "UserName": "Имя пользователя",
                "Manufacturer": "Производитель", "Name": "Название", "Product": "Изделие",
                "MaxClockSpeed": "Максимальная тактовая частота", "SocketDesignation": "Название сокета",
                "NumberOfPhysicalProcessors": "Количество физических процессоров", "VideoProcessor": "Видеопроцессор",
                "NumberOfLogicalProcessors": "Количество логических процессоров", "Capacity": "Емкость",
                "AdapterRAM": "Оперативная память адаптера", "CurrentRefreshRate": "Текущая частота обновления",
                "Resolution": "Разрешение", "TotalPhysicalMemory": "Общий объем физической памяти",
                "ConfiguredClockSpeed": "Настроенная тактовая частота", "PartNumber": "Номер партии",
                "SerialNumber": "Серийный номер", "DeviceID": "Идентификатор устройства", "MediaType": "Тип носителя",
                "FirmwareRevision": "Ревизия прошивки", "Partitions": "Разделы", "Size": "Объем", "Drive": "Диск",
                "VolumeName": "Имя тома", "VolumeSerialNumber": "Серийный номер тома", "MACAddress": "MAC-адрес",
                "NetConnectionID": "Идентификатор сетевого подключения", "DHCPServer": "DHCP-сервер",
                "IPAddress": "IP-адрес"}
    part += f'{"-" * 50}\n'
    for key in dict_info:
        if type(dict_info[key]) == dict:
            for item in dict_info[key]:
                part += f'{synonyms[item]}: {dict_info[key][item]}\n'
            part += "\n"
        else:
            part += f'{synonyms[key]}: {dict_info[key]}\n'
    print(part)
    wmic_info += f'{part}\n'


def main():
    global wmic_info
    t = time.monotonic()
    document = Document()
    document.add_heading(f'Сводная информация о компьютере: {node()}', 0)

    if os_info := wmic_os():
        print_wmic("Информация об операционной системе\n", os_info)
    if bios_info := bios_wmic():
        print_wmic("Информация о BIOS\n", bios_info)
    if mb_info := motherboard_wmic():
        print_wmic("Информация о материнской плате\n", mb_info)
    if cpu_info := cpu_wmic():
        print_wmic("Информация о процессоре\n", cpu_info)
    if gpu_info := gpu_wmic():
        print_wmic("Информация о видеокарте\n", gpu_info)
    if mem_info := memory_wmic():
        print_wmic("Информация об оперативной памяти\n", mem_info)
    if drive_info := hdd_ssd_wmic():
        print_wmic("Информация о HDD и SSD\n", drive_info)
    if cd_rom_info := cdrom_wmic():
        print_wmic("Информация о CD/DVD-ROM\n", cd_rom_info)
    if nic_info := nic_wmic():
        print_wmic("Информация о физических сетевых интерфейсах\n", nic_info)

    document.add_paragraph(wmic_info)
    document.save(f'{node()}.docx')
    print(f"Собранная информация сохранена в файл: {node()}.docx")
    print(f'\nВремя работы скрипта: {time.monotonic() - t} с.')


if __name__ == "__main__":
    main()
